/**
 * This project contains classes and equality tests for 
 *      'Overriding equals() - a simple solution'.
 */

package equality;

/**
 *
 * @author Martin Humby
 */
public class Equality {
    
    public static void eqPrint(Object a, Object b) {
        String ac = a.getClass().getCanonicalName();
        if (ac == null) ac = "?"; else ac = 
            ac.substring(ac.indexOf('.') + 1);
        String bc = b.getClass().getCanonicalName();
        if (bc == null) bc = "?"; else bc = 
            bc.substring(bc.indexOf('.') + 1);
        System.out.println(ac + " equals() " + bc + ": " +
            a.equals(b));
        System.out.println(bc + " equals() " + ac + ": " +
            b.equals(a));
        System.out.println("Hashcodes are equal: " +
                              (a.hashCode() == b.hashCode()));
    }
    
    public static void println () {
        System.out.println();
    }
    
    public static void println (Object o) {
        System.out.println(o);
    }
    
    public static void checkCreated(Object o) {
        String ac = o.getClass().getCanonicalName();
        if (ac == null) ac = "?"; else ac = 
            ac.substring(ac.indexOf('.') + 1);
        println(ac + " created");
    }
    
    public static void main(String[] args) {
        println ("Create instances of standard classes:");
        RegisteredItem ri = new RegisteredItem("itemId", "catergoryID");
        checkCreated(ri);
        DataItem di = new DataItem("itemId", "catergoryID");
        checkCreated(di);
        RegionItem rgi = new RegionItem(1, "itemId", "catergoryID");
        checkCreated(rgi);
        RegionItem rdi = new RgnDataItem(1, "itemId", "catergoryID");
        checkCreated(rdi);
        println("-------------------------------------------------------------------------");
        println("Check equality of a DataItem with a RegisteredItem having the same field values");
        eqPrint(di, ri);        
        println("-------------------------------------------------------------------------");
        println("and for a RgnDataItem and a RegionItem");
        eqPrint(rdi, rgi); 
        println("\nequals() has not been overriden and equality is maintained"); 
        println();      
        
        println("-------------------------------------------------------------------------");        
        println("However for a RegionItem and a RegisteredItem");
        eqPrint(rgi, ri);        
        println("-------------------------------------------------------------------------");
        println("a RgnDataItem and a RegisteredItem");
        eqPrint(rdi, ri);        
        println("-------------------------------------------------------------------------");
        println("or a RgnDataItem and DataItem");
        eqPrint(rdi, di); 
        println("\nequals() has been overriden but symmetry is maintained in all cases");
        println();    
        
        println("-------------------------------------------------------------------------");
        println("Now create some objects with equality of the third kind:");
        RegionItem3 rgi3 = new RegionItem3(0, "itemId", "catergoryID");
        checkCreated(rgi3);
        println("regionID is " + rgi3.getRegionID());
        RgnDataItem3 rdi3 = new RgnDataItem3(0, "itemId", "catergoryID");
        checkCreated(rdi3);
        println("regionID is " + rdi3.getRegionID());        
        println("-------------------------------------------------------------------------");
        println("Comparing these objects with a RegisteredItem");
        eqPrint(rgi3, ri);  
        println();          
        eqPrint(rdi3, ri);        
        println("-------------------------------------------------------------------------");        
        println("But when objets are created with regionID of 1");
        RegionItem3 rgi31 = new RegionItem3(1, "itemId", "catergoryID");
        checkCreated(rgi31);
        println("regionID is " + rgi31.getRegionID());
        RgnDataItem3 rdi31 = new RgnDataItem3(1, "itemId", "catergoryID");
        checkCreated(rdi31);
        println("regionID is " + rdi31.getRegionID());         
        println("-------------------------------------------------------------------------");
        println("Comparing these objects with a RegisteredItem");
        eqPrint(rgi31, ri);  
        println();          
        eqPrint(rdi31, ri);          
        
        
    }
    
}
